package com.anplus.admincontrol.security.authentication.filter;


import com.anplus.common.admin.security.authentication.AccountCredentials;
import com.anplus.common.admin.security.authentication.LoginSuccessDTO;
import com.anplus.common.admin.security.authentication.TokenService;
import com.core.result.Result;
import com.core.result.ResultCodeEnum;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.InternalAuthenticationServiceException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Collections;

/**
 * @Author wangning
 * @Date 2019-10-04 17:22
 * @Email 1151089590@qq.com
 * @Description:
 */
public class SecurityLoginFilter extends AbstractAuthenticationProcessingFilter {

    /**
     * 默认一天有效时常 秒
     */
    @Value("${project.jwt.token-expire-time}")
    public long tokenExpireTime = 86_400;
    /**
     * 默认使用记住我模式10天token有效时长 秒
     */
    @Value("${project.jwt.token-expire-time-long}")
    public long tokenExpireTimeLong = 864_000;

    private ObjectMapper objectMapper;

    private TokenService tokenService;

    /**
     * 满足url ant匹配条件时进行校验
     *
     * @param url
     * @param authManager
     */
    public SecurityLoginFilter(String url, AuthenticationManager authManager, TokenService tokenService, ObjectMapper objectMapper) {
        super(new AntPathRequestMatcher(url));
        setAuthenticationManager(authManager);
        this.objectMapper = objectMapper;
        this.tokenService = tokenService;
    }

    /**
     * 解析出用户名密码交给AuthenticationManager进行验证
     *
     * @param req
     * @param res
     * @return
     * @throws AuthenticationException
     * @throws IOException
     * @throws ServletException
     */
    @Override
    public Authentication attemptAuthentication(HttpServletRequest req, HttpServletResponse res)
            throws AuthenticationException, IOException {
        String method = req.getMethod();
        System.out.println(method);
        AccountCredentials credentials = objectMapper.readValue(req.getInputStream(), AccountCredentials.class);
        req.setAttribute("rememberMe", credentials.getRememberMe());
        return getAuthenticationManager().authenticate(
                new UsernamePasswordAuthenticationToken(
                        credentials.getUsername(),
                        credentials.getPassword(),
                        Collections.emptyList()
                )
        );
    }

    /**
     * 登录成功
     *
     * @param req
     * @param res
     * @param chain
     * @param auth
     * @throws IOException
     * @throws ServletException
     */
    @Override
    protected void successfulAuthentication(
            HttpServletRequest req,
            HttpServletResponse res, FilterChain chain,
            Authentication auth) throws IOException {
        Boolean rememberMe = (Boolean) req.getAttribute("rememberMe");
        rememberMe = rememberMe == null ? false : rememberMe;
        String token = tokenService.addAuthentication(auth.getName(), rememberMe);
        res.setContentType("application/json;charset=UTF-8");
        Result result = new Result();
        result.setCode(ResultCodeEnum.SUCCESS);
        result.setMsg("登录成功！");
        LoginSuccessDTO loginSuccessDTO = new LoginSuccessDTO();
        loginSuccessDTO.setToken(token);
        loginSuccessDTO.setExpirationTime(rememberMe ? tokenExpireTimeLong : tokenExpireTime);
        result.setData(loginSuccessDTO);
        res.getWriter().write(objectMapper.writeValueAsString(result));
    }

    /**
     * 登录失败
     *
     * @param req
     * @param res
     * @param exception
     * @throws IOException
     * @throws ServletException
     */
    @Override
    protected void unsuccessfulAuthentication(
            HttpServletRequest req,
            HttpServletResponse res,
            AuthenticationException exception) throws IOException {
        res.setContentType("application/json;charset=UTF-8");
        Result result = new Result();
        result.setCode(ResultCodeEnum.UNAUTHORIZED);
        if (exception instanceof InternalAuthenticationServiceException) {
            result.setMsg(exception.getMessage());
        } else {
            result.setMsg("用户名或密码错误！");
        }
        res.getWriter().write(objectMapper.writeValueAsString(result));
    }

}
