package com.anplus.admincontrol.service.holidays;

import com.anplus.admincontrol.controller.holidays.dto.HolidaysPageDTO;
import com.anplus.admincontrol.mapper.holidays.HolidaysBusinessMapper;
import com.anplus.admincontrol.mapper.holidays.bo.HolidayBO;
import com.common.http.HttpUtils;
import com.constants.CodeConstants;
import com.core.exception.ServiceException;
import com.dao.mapper.holidays.HolidaysMapper;
import com.dao.model.holidays.Holidays;
import com.dao.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Condition;

import javax.annotation.Resource;
import java.text.MessageFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class HolidaysService {

    @Value("${holidays.url}")
    private String url;

    @Value(("${holidays.key}"))
    private String key;


    @Resource
    HolidaysMapper holidaysMapper;

    @Resource
    HolidaysBusinessMapper holidaysBusinessMapper;

    /**
     * 一年节假日列表
     */
    public List<Holidays> holidaysList(Long year,Long userId) throws ParseException {

        if(year<1970 || year >9999){
            throw new ServiceException("年数异常");
        }

        String requestUrl = url+ "?key="+ key + "&date=" + year +"&type=" + 1;

        List<Holidays> holidays = new ArrayList<>();

        ResponseEntity responseEntity = HttpUtils.get(requestUrl, Map.class);

        Map map = (HashMap) responseEntity.getBody();

        if (responseEntity.getStatusCode().value() != 200){
            throw new ServiceException(MessageFormat.format("{0}接口请求异常，异常信息:{1}",requestUrl,map.get("msg")));
        }
            if((Integer)map.get("code")==250){
                throw new ServiceException("暂无该年度节假日数据");
            }
            List<HashMap> newList = (List<HashMap>)map.get("newslist");

            for (HashMap item:newList) {

                String name = (String) item.get("name");

                String[] vacationsArr = ((String) item.get("vacation")).split("\\|");
                String[] remarksArr = ((String) item.get("remark")).split("\\|");

                List<String> remarks = new ArrayList<>();
                List<String> vacations = new ArrayList<>();

                for (String s:remarksArr) {
                    remarks.add(s);
                }
                for (String s:vacationsArr) {
                    vacations.add(s);
                }

                //去除空白字符串
                remarks.stream().filter(string -> !string.isEmpty()).collect(Collectors.toList());
                vacations.stream().filter(string -> !string.isEmpty()).collect(Collectors.toList());


                SimpleDateFormat simdf = new SimpleDateFormat("yyyy-MM-dd");
                Date startDate = simdf.parse(vacationsArr[0]);
                Date endDate = simdf.parse(vacationsArr[vacationsArr.length-1]);

                Integer duration = vacations.size();

                Holidays holiday = Holidays.builder()
                        .name(name)
                        .dayCode(CodeConstants.HolidayType.HOLIDAY)
                        .startDate(startDate)
                        .endDate(endDate)
                        .year(year)
                        .startMonth(startDate.getMonth()+1)
                        .endMonth(endDate.getMonth()+1)
                        .vacation(vacations.toString().replace("[","").replace("]",""))
                        .duration(duration.longValue())
                        .vacations(vacations)
                        .remarks(remarks)
                        .build();
                EntityUtils.setBase(holiday,userId);

                holidays.add(holiday);
            }

        return holidays;
    }

    /**
     * 一年双休日列表
     */
    public List<Holidays> weekendList(Long year,List<Holidays> holidaysList,Long userId){

        List<String> excludeDays = new ArrayList<>();

        holidaysList.stream().forEach(item->{
            excludeDays.addAll(item.getVacations());
            excludeDays.addAll(item.getRemarks());
        });

        if(year<1970 || year >9999){
            throw new ServiceException("年数异常");
        }

        Calendar calendar = new GregorianCalendar(year.intValue(), 0, 1);

        List<Holidays> holidays=new ArrayList<>();
        SimpleDateFormat simdf = new SimpleDateFormat("yyyy-MM-dd");

        int index = 1;
        while (calendar.get(Calendar.YEAR)<year+1){
            calendar.set(Calendar.WEEK_OF_YEAR, index++);

            //周日
            calendar.set(Calendar.DAY_OF_WEEK, Calendar.SUNDAY);
            if (calendar.get(Calendar.YEAR) == year) {
                String day = simdf.format(calendar.getTime());
                if(!excludeDays.contains(day)){
                    Date startDate = null;
                    try {
                        startDate = simdf.parse(day);
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }

                    Holidays holiday = Holidays.builder()
                            .name("星期日")
                            .dayCode(CodeConstants.HolidayType.WEEKEND)
                            .startDate(startDate)
                            .endDate(startDate)
                            .year(year)
                            .startMonth(startDate.getMonth()+1)
                            .endMonth(startDate.getMonth()+1)
                            .vacation(day)
                            .duration(1L)
                            .build();
                    EntityUtils.setBase(holiday,userId);
                    holidays.add(holiday);
                }
            }

            //周六
            calendar.set(Calendar.DAY_OF_WEEK, Calendar.SATURDAY);
            if (calendar.get(Calendar.YEAR) == year) {
                String day = simdf.format(calendar.getTime());
                if(!excludeDays.contains(day)){
                    Date startDate = null;
                    try {
                        startDate = simdf.parse(day);
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }

                    Holidays holiday = Holidays.builder()
                            .name("星期六")
                            .dayCode(CodeConstants.HolidayType.WEEKEND)
                            .startDate(startDate)
                            .endDate(startDate)
                            .year(year)
                            .startMonth(startDate.getMonth()+1)
                            .endMonth(startDate.getMonth()+1)
                            .vacation(day)
                            .duration(1L)
                            .build();
                    EntityUtils.setBase(holiday,userId);
                    holidays.add(holiday);
                }
            }
        }


        return holidays;
    }

    /**
     * 节假日合并
     */
    public List<Holidays> mergeHolidays(List<Holidays> holidays) throws ParseException {
        for (int i=0;i<holidays.size()-1;i++){
            int size = holidays.get(i).getVacations().size();
            String previous = holidays.get(i).getVacations().get(size-1);
            String next = holidays.get(i+1).getVacations().get(0);
            if(vacationAdjacent(previous,next)){
                Holidays temp = merge(holidays.get(i),holidays.get(i+1));
                holidays.set(i,temp);
                holidays.remove(i+1);
                i-=1;
            }
        }
        return holidays;
    }


    /**
     * 同步节假日
     */
    public void synchronousVacation(HolidaysPageDTO param,Long userId) throws ParseException {

        if(hadSynchronousVacation(param.getYear())){
            throw new ServiceException("当前年度已同步假期");
        }

        List<Holidays> holidaysList = holidaysList(param.getYear(),userId);
        holidaysList = mergeHolidays(holidaysList);
        holidaysList.addAll(weekendList(param.getYear(),holidaysList,userId));

        holidaysMapper.insertListSelective(holidaysList);
    }

    /**
     * 分页获取列表
     * @param param
     * @return
     */
    public List<Holidays> getPage(HolidaysPageDTO param){
        return holidaysBusinessMapper.getPage(param);
    }

    /**
     * 删除假期
     * @param id
     */
    public void deleteById(Long id){
        holidaysMapper.deleteByPrimaryKey(id);
    }

    /**
     * 批量删除假期
     * @param ids
     */
    public void deleteByIds(List<Long> ids){
        holidaysMapper.deleteByIdList(ids);
    }

    /**
     *  新增节假日
     */
    public void addAndUpdateHoliday(Holidays holidays,Long userId){

        Date startDate = holidays.getStartDate();
        Date endDate = holidays.getEndDate();

        Boolean isOverlap = holidaysOverlap(holidays);

        if(isOverlap){
            throw  new ServiceException("假期时间存在重叠，请重新选择假期时间");
        }

        // 更新
        if(holidays.getId()!=null){
            EntityUtils.setModifyBase(holidays,userId);
            holidays.setVacation(getBetweenDates(startDate,endDate));
            holidaysMapper.updateByPrimaryKeySelective(holidays);
        }
        // 新增
        else{
            Holidays holiday = Holidays.builder()
                    .name(holidays.getName())
                    .dayCode(holidays.getDayCode())
                    .startDate(startDate)
                    .endDate(endDate)
                    .year(holidays.getYear())
                    .startMonth(startDate.getMonth()+1)
                    .endMonth(endDate.getMonth()+1)
                    .vacation(getBetweenDates(startDate,endDate))
                    .duration(holidays.getDuration())
                    .build();
            EntityUtils.setBase(holiday,userId);

            holidaysMapper.insert(holiday);
        }
    }

    /**
     * 判断假期是否相邻
     * @param previous
     * @param next
     * @return
     */
    private Boolean vacationAdjacent(String previous,String next){

        SimpleDateFormat simdf = new SimpleDateFormat("yyyy-MM-dd");

        Date previousDate = null;
        Date nextDate = null;

        try {
            previousDate = simdf.parse(previous);
            nextDate = simdf.parse(next);
        }catch (Exception e){
            throw new ServiceException("日期转换失败");
        }

        Calendar previousCalendar = Calendar.getInstance();
        previousCalendar.setTime(previousDate);

        Calendar nextCalendar = Calendar.getInstance();
        nextCalendar.setTime(nextDate);

        previousCalendar.add(Calendar.DATE,1);

        //假期相邻
        return ((previousDate.compareTo(nextDate) == 0) || (previousCalendar.compareTo(nextCalendar)==0));
    }

    /**
     * 合并假期
     * @param previous
     * @param next
     * @return
     */
    private Holidays merge(Holidays previous,Holidays next) throws ParseException {
        previous.setName(previous.getName().replace("节","")+","+next.getName());

        List<String> previousVavations = previous.getVacations();

        previousVavations.removeAll(next.getVacations());
        previousVavations.addAll(next.getVacations());

        List<String> previousRemarks = previous.getRemarks();

        previousRemarks.removeAll(next.getRemarks());
        previousRemarks.addAll(next.getRemarks());

        previous.setVacations(previousVavations);
        previous.setRemarks(previousRemarks);

        previous.setVacation(previousVavations.toString().replace("[","").replace("]",""));
        Integer size = previousVavations.size();
        previous.setDuration(size.longValue());

        SimpleDateFormat simdf = new SimpleDateFormat("yyyy-MM-dd");
        Date endDate = simdf.parse(previousVavations.get(previousVavations.size()-1));
        previous.setEndDate(endDate);
        previous.setEndMonth(endDate.getMonth()+1);

        return previous;
    }

    /**
     * 判断当前年度假期是否已同步
     * @param year
     * @return
     */
    private Boolean hadSynchronousVacation(Long year){
        Condition condition = new Condition(Holidays.class);
        condition.createCriteria()
                .andEqualTo("year",year)
                .andEqualTo("deleted",CodeConstants.DeletedStatus.UNDELETED);

       return holidaysMapper.selectCountByCondition(condition) > 1;
    }

    /**
     * 获取两个日期之间的所有日期
     * @param start
     * @param end
     * @return
     */
    private String getBetweenDates(Date start, Date end) {

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");

        if(start.compareTo(end)==0){
            return simpleDateFormat.format(start);
        }

        List<String> result = new ArrayList<>();
        Calendar tempStart = Calendar.getInstance();
        result.add(simpleDateFormat.format(start));
        tempStart.setTime(start);
        tempStart.add(Calendar.DAY_OF_YEAR, 1);

        Calendar tempEnd = Calendar.getInstance();
        tempEnd.setTime(end);
        while (tempStart.before(tempEnd)) {
            result.add(simpleDateFormat.format(tempStart.getTime()));
            tempStart.add(Calendar.DAY_OF_YEAR, 1);
        }
        result.add(simpleDateFormat.format(end));
        return result.toString().replace("[","").replace("]","");
    }

    /**
     * 是否存在重叠假期
     * @param holidays
     * @return
     */
    private Boolean holidaysOverlap(Holidays holidays){

        // AstartDate >= BstartDate && AstartDate<=BEndDate  || AendDate >= BstartDate && AendDate <= BendDate  判断假期重复逻辑

        if(holidays.getStartDate()==null || holidays.getEndDate()==null){
            throw new ServiceException("输入日期为空");
        }

        Boolean isover =   holidaysMapper.holidaysOverlap(holidays.getStartDate(),holidays.getEndDate(),holidays.getId()) > 0;

        return isover;
    }

    /**
     * 获取某一年的节假日，如果是今年，只查询到当年到当日的节假日
     *      * 此查询无法用于未来的年份！
     * @param year 例 2015
     */
    public List<HolidayBO> getByYear(Integer year) {
        return holidaysBusinessMapper.getByYear(year);
    }
}
