package com.anplus.admincontrol.service.sys;

import com.anplus.admincontrol.constant.ErrorConstant;
import com.anplus.admincontrol.controller.sys.dto.SysUserDetailDTO;
import com.anplus.admincontrol.controller.sys.dto.SysUserPageDTO;
import com.anplus.admincontrol.controller.sys.vo.SysUserDetailVO;
import com.anplus.admincontrol.mapper.sys.SysUserCacheMapper;
import com.anplus.admincontrol.mapper.sys.SysUserMapper;
import com.common.util.CommonUtils;
import com.core.exception.ServiceException;
import com.dao.common.AbstractMapperService;
import com.dao.model.sys.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author wangning
 * @date 2019-10-11 09:02
 * SysUserService服务类
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class SysUserService extends AbstractMapperService<SysUser> {

    @Value("${project.default-pwd}")
    private String defaultPwd;

    @Resource
    private SysUserMapper sysUserMapper;

    @Autowired
    private SysUserCacheMapper sysUserCacheMapper;

    @Autowired
    private SysRoleService sysRoleService;

    @Autowired
    private SysUserRoleService sysUserRoleService;

    @Autowired
    private BCryptPasswordEncoder bCryptPasswordEncoder;

    /**
     * 分页查询用户数据
     *
     * @param dto
     * @return java.util.List<com.dao.model.sys.SysUser>
     * @author wangning
     * @date 2019-11-14 11:57
     */
    public List<SysUser> getPage(SysUserPageDTO dto) {
        return sysUserMapper.getPage(dto);
    }

    /**
     * 用户详情页查询
     *
     * @param id
     * @return
     */
    public SysUserDetailVO getUserDetail(int id) {
        SysUserDetailVO result = new SysUserDetailVO();
        if (CommonUtils.isZero(id)) {
            result.setSysUserDetail(new SysUser());
        } else {
            result.setSysUserDetail(selectByPrimaryKey(id));
            result.setSysUserCheckedRoleList(sysUserRoleService.getUserRoleByUserId(id));
        }
        result.setSysUserRoleList(sysRoleService.getAllAvailableRole());
        return result;
    }

    /**
     * 保存用户信息
     *
     * @param dto
     * @return
     */
    public long saveDetail(SysUserDetailDTO dto) {
        //待保存用户信息
        SysUser newUserInfo = dto.getSysUserDetail();

        // 校验用户名是否重复
        if (sysUserMapper.getRepeat(CommonUtils.toInteger(newUserInfo.getId()), newUserInfo.getUsername()) > 0) {
            throw new ServiceException(ErrorConstant.USERNAME_EXIST);
        }

        // 勾选角色
        List<Integer> sysUserCheckedRoleList = dto.getSysUserCheckedRoleList();

        // 根据数据有无id判断是新建还是修改
        if (CommonUtils.isZero(newUserInfo.getId())) {
            setSysUserDefaultInfo(newUserInfo);
            sysUserMapper.insertSelective(newUserInfo);
        } else {
            // 更新，同时清空redis
            sysUserCacheMapper.updateByPrimaryKey(newUserInfo);
        }

        // 先删除
        sysUserRoleService.deleteByUserId(newUserInfo.getId());

        if (!CommonUtils.isEmpty(sysUserCheckedRoleList)) {
            // 再增加
            sysUserRoleService.insertUserRole(newUserInfo.getId(), sysUserCheckedRoleList);
        }
        return newUserInfo.getId();
    }

    /**
     * 优先使用缓存
     *
     * @param user
     * @return void
     * @author wangning
     * @date 2019-10-14 9:27
     */
    public void saveCache(SysUser user) {
        sysUserCacheMapper.updateByPrimaryKey(user);
    }

    /**
     * 查询用户，优先使用缓存
     *
     * @param username
     * @return com.dao.model.sys.SysUser
     * @author wangning
     * @date 2019-10-14 10:38
     */
    public SysUser getCatchByUserName(String username) {
        return sysUserCacheMapper.getByUserName(username);
    }

    /**
     * 重置用户密码
     *
     * @param id
     * @return void
     * @author wangning
     * @date 2019-10-14 11:30
     */
    public void resetPwd(int id) {
        SysUser sysUser = selectByPrimaryKey(id);
        sysUser.setPassword(bCryptPasswordEncoder.encode(defaultPwd));
        // 更新同时清空redis
        saveCache(sysUser);
    }

    /**
     * 修改密码
     *
     * @param paramSysUser
     * @param loginUser
     * @return void
     * @author wangning
     * @date 2019-10-14 11:54
     */
    public void editPwd(SysUser paramSysUser, SysUser loginUser) {

        if (CommonUtils.isEmpty(paramSysUser.getNewPassword()) || CommonUtils.isEmpty(paramSysUser.getConfirmNewPassword())) {
            throw new ServiceException(ErrorConstant.PASSWORD_IS_EMPTY);
        } else {
            if (!paramSysUser.getNewPassword().equals(paramSysUser.getConfirmNewPassword())) {
                throw new ServiceException(ErrorConstant.NEW_PASSWORD_ENTERED_TWICE_IS_INCONSISTENT);
            }
        }
        long userId = loginUser.getId();
        SysUser user = selectByPrimaryKey((int) userId);
        if (bCryptPasswordEncoder.matches(paramSysUser.getPassword(), user.getPassword())) {
            user.setPassword(bCryptPasswordEncoder.encode(paramSysUser.getNewPassword()));
            // 保存，并清空redis
            saveCache(user);
        } else {
            throw new ServiceException(ErrorConstant.PASSWORD_ERROR);
        }
    }

    /**
     * 修改用户状态
     *
     * @param id
     * @return java.lang.Long
     * @author wangning
     * @date 2019-11-14 11:58
     */
    public Long changeStatus(Long id) {
        SysUser sysUser = selectByPrimaryKey(CommonUtils.toInteger(id));
        sysUser.setAvailable(!sysUser.getAvailable());
        // 更新同时清空redis
        saveCache(sysUser);
        return sysUser.getId();
    }

    /**
     * 设置SysUser默认信息
     *
     * @param sysUser
     */
    private void setSysUserDefaultInfo(SysUser sysUser) {
        // 默认密码
        sysUser.setPassword(bCryptPasswordEncoder.encode(defaultPwd));
        // 默认启用状态
        sysUser.setAvailable(true);
    }
}
