package com.common.http;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import java.net.ProxySelector;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.time.Duration;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

/**
 * @author 潘维吉
 * @date 2019-10-29
 * HttpClient网络请求工具类 基于JDK11默认实现
 */
@Service
@Slf4j
public class HttpClientUtils {

    private static final String CHARSET = "UTF-8"; //编码
    private static final int TIMEOUT = 60000; //超时时间
    private HttpClient client;

    @PostConstruct
    private void init() {
        try {
            // 创建SSLContext对象，并使用我们指定的信任管理器初始化
            TrustManager[] tm = {new HttpsTrustManager()};
            SSLContext sslContext = SSLContext.getInstance("SSL", "SunJSSE");
            sslContext.init(null, tm, new java.security.SecureRandom());

            client = HttpClient.newBuilder()
                    .version(HttpClient.Version.HTTP_2) // default
                    .followRedirects(HttpClient.Redirect.NORMAL) ///Always redirect, except from HTTPS URLs to HTTP URLs
                    .proxy(ProxySelector.getDefault())
                    .connectTimeout(Duration.ofMillis(5000))
                    .sslContext(sslContext)
                    .build();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * GET同步发送请求
     */
    public Object get(String url) {
        return getSend(url, true, "");
    }

    /**
     * GET异步发送请求
     */
    public Object getAsync(String url) {
        return getSend(url, false, "");
    }

    /**
     * GET同步发送请求 带token
     */
    public Object get(String url, String token) {
        return getSend(url, true, token);
    }

    /**
     * GET异步发送请求 带token
     */
    public Object getAsync(String url, String token) {
        return getSend(url, false, token);
    }

    /**
     * POST同步发送请求
     */
    public Object post(String url, Map body) {
        return postSend(url, body, true, "");
    }

    /**
     * POST异步发送请求
     */
    public Object postAsync(String url, Map body) {
        return postSend(url, body, false, "");
    }

    /**
     * POST同步发送请求 带token
     */
    public Object post(String url, Map body, String token) {
        return postSend(url, body, true, token);
    }

    /**
     * POST异步发送请求 带token
     */
    public Object postAsync(String url, Map body, String token) {
        return postSend(url, body, false, token);
    }


    /**
     * Put同步发送请求
     */
    public Object put(String url, Map body) {
        return putSend(url, body, true, "");
    }

    /**
     * Put异步发送请求
     */
    public Object putAsync(String url, Map body) {
        return putSend(url, body, false, "");
    }

    /**
     * Put同步发送请求 带token
     */
    public Object put(String url, Map body, String token) {
        return putSend(url, body, true, token);
    }

    /**
     * Put异步发送请求 带token
     */
    public Object putAsync(String url, Map body, String token) {
        return putSend(url, body, false, token);
    }

    /**
     * Delete同步发送请求
     */
    public Object delete(String url) {
        return deleteSend(url, true, "");
    }

    /**
     * Delete异步发送请求
     */
    public Object deleteAsync(String url) {
        return deleteSend(url, false, "");
    }

    /**
     * Delete同步发送请求 带token
     */
    public Object delete(String url, String token) {
        return deleteSend(url, true, token);
    }

    /**
     * Delete异步发送请求 带token
     */
    public Object deleteAsync(String url, String token) {
        return deleteSend(url, false, token);
    }

    /**
     * GET发送请求
     *
     * @param url    请求url
     * @param isSync 是否是同步
     * @param token  鉴权token
     * @return
     */
    private Object getSend(String url, Boolean isSync, String token) {
        HttpRequest request = HttpRequest.newBuilder()
                .uri(URI.create(url))
                .header("Authorization", token)
                .timeout(Duration.ofMillis(TIMEOUT))
                .build();

        HttpResponse<String> response = null;
        CompletableFuture<String> result = null;
        try {
            if (isSync) {
                response = client.send(request, HttpResponse.BodyHandlers.ofString());
                return response.body();
            } else {
                result = client.sendAsync(request, HttpResponse.BodyHandlers.ofString())
                        .thenApply(HttpResponse::body);
                return result.get();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * POST JSON发送请求
     *
     * @param url    请求url
     * @param body   数据
     * @param isSync 是否是同步
     * @param token  鉴权token
     * @return
     */
    private Object postSend(String url, Map body, Boolean isSync, String token) {
        ObjectMapper objectMapper = new ObjectMapper();
        String requestBody = null;
        try {
            requestBody = objectMapper
                    .writerWithDefaultPrettyPrinter()
                    .writeValueAsString(body);

            HttpRequest request = HttpRequest.newBuilder()
                    .uri(URI.create(url))
                    .header("Authorization", token)
                    .header("Content-Type", "application/json")
                    .POST(HttpRequest.BodyPublishers.ofString(requestBody))
                    .timeout(Duration.ofMillis(TIMEOUT))
                    .build();

            HttpResponse<String> response = null;
            CompletableFuture<String> result = null;
            if (isSync) {
                response = client.send(request, HttpResponse.BodyHandlers.ofString());
                return response.body();
            } else {
                result = client.sendAsync(request, HttpResponse.BodyHandlers.ofString())
                        .thenApply(HttpResponse::body);
                return result.get();

            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    /**
     * PUT JSON发送请求
     *
     * @param url    请求url
     * @param body   数据
     * @param isSync 是否是同步
     * @param token  鉴权token
     * @return
     */
    private Object putSend(String url, Map body, Boolean isSync, String token) {
        ObjectMapper objectMapper = new ObjectMapper();
        String requestBody = null;
        try {
            requestBody = objectMapper
                    .writerWithDefaultPrettyPrinter()
                    .writeValueAsString(body);

            HttpRequest request = HttpRequest.newBuilder()
                    .uri(URI.create(url))
                    .header("Authorization", token)
                    .header("Content-Type", "application/json")
                    .PUT(HttpRequest.BodyPublishers.ofString(requestBody))
                    .timeout(Duration.ofMillis(TIMEOUT))
                    .build();

            HttpResponse<String> response = null;
            CompletableFuture<String> result = null;
            if (isSync) {
                response = client.send(request, HttpResponse.BodyHandlers.ofString());
                return response.body();
            } else {
                result = client.sendAsync(request, HttpResponse.BodyHandlers.ofString())
                        .thenApply(HttpResponse::body);
                return result.get();

            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    /**
     * Delete发送请求
     *
     * @param url    请求url
     * @param isSync 是否是同步
     * @param token  鉴权token
     * @return
     */
    private Object deleteSend(String url, Boolean isSync, String token) {
        HttpRequest request = HttpRequest.newBuilder()
                .uri(URI.create(url))
                .DELETE()
                .header("Authorization", token)
                .timeout(Duration.ofMillis(TIMEOUT))
                .build();

        HttpResponse<String> response = null;
        CompletableFuture<String> result = null;
        try {
            if (isSync) {
                response = client.send(request, HttpResponse.BodyHandlers.ofString());
                return response.body();
            } else {
                result = client.sendAsync(request, HttpResponse.BodyHandlers.ofString())
                        .thenApply(HttpResponse::body);
                return result.get();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

}
