package com.common.util;

import org.apache.commons.lang3.time.DateFormatUtils;

import javax.annotation.Nonnull;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

/**
 * @author 潘维吉
 * @date 2019/10/28 14:19
 * @email 406798106@qq.com
 * @description 日期时间工具类
 */
public class DateUtils extends org.apache.commons.lang3.time.DateUtils {

    private static String[] parsePatterns = {"yyyy-MM-dd", "yyyy-MM", "yyyy年MM月",
            "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", "yyyy/MM/dd",
            "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm", "yyyyMMdd",
            "yyyy-MM-dd-HH-mm-ss"};

    /**
     * 得到当前日期字符串 格式（yyyy-MM-dd）
     */
    public static String getDate() {
        return getDate("yyyy-MM-dd");
    }

    /**
     * 得到当前日期字符串 格式（yyyy-MM-dd） pattern可以为："yyyy-MM-dd" "HH:mm:ss" "E"
     */
    public static String getDate(String pattern) {
        return DateFormatUtils.format(new Date(), pattern);
    }

    /**
     * 得到日期字符串 默认格式（yyyy-MM-dd） pattern可以为："yyyy-MM-dd" "HH:mm:ss" "E"
     */
    public static String formatDate(Date date, Object... pattern) {
        String formatDate = null;
        if (pattern != null && pattern.length > 0) {
            formatDate = DateFormatUtils.format(date, pattern[0].toString());
        } else {
            formatDate = DateFormatUtils.format(date, "yyyy-MM-dd");
        }
        return formatDate;
    }

    /**
     * 得到日期时间字符串，转换格式（yyyy-MM-dd HH:mm:ss）
     */
    public static String formatDateTime(Date date) {
        return formatDate(date, "yyyy-MM-dd HH:mm:ss");
    }

    /**
     * 得到日期时间字符串，转换格式（yyyy-MM-dd HH:mm）
     */
    public static String formatDateTimeMinute(Date date) {
        return formatDate(date, "yyyy-MM-dd HH:mm");
    }

    /**
     * 得到日期时间字符串，转换格式（yyyy-MM-dd）
     */
    public static String formatDateTimeDay(Date date) {
        return formatDate(date, "yyyy-MM-dd");
    }

    /**
     * 得到当前时间字符串 格式（HH:mm:ss）
     */
    public static String getTime() {
        return formatDate(new Date(), "HH:mm:ss");
    }

    /**
     * 得到当前日期和时间字符串 格式（yyyy-MM-dd HH:mm:ss）
     */
    public static String getDateTime() {
        return formatDate(new Date(), "yyyy-MM-dd HH:mm:ss");
    }

    /**
     * 得到当前年份字符串 格式（yyyy）
     */
    public static String getYear() {
        return formatDate(new Date(), "yyyy");
    }

    /**
     * 得到当前月份字符串 格式（MM）
     */
    public static String getMonth() {
        return formatDate(new Date(), "MM");
    }

    /**
     * 得到当天字符串 格式（dd）
     */
    public static String getDay() {
        return formatDate(new Date(), "dd");
    }

    /**
     * 得到当天字符串 格式（dd）
     */
    public static String getDay(Date date) {
        return formatDate(date, "dd");
    }

    /**
     * 得到当前星期字符串 格式（E）星期几
     */
    public static String getWeek() {
        return formatDate(new Date(), "E");
    }

    /**
     * 日期型字符串转化为日期 格式 { "yyyy-MM-dd", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm",
     * "yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm", "yyyyMMdd" }
     */
    public static Date parseDate(Object str) {
        if (str == null) {
            return null;
        }
        try {
            return parseDate(str.toString(), parsePatterns);
        } catch (ParseException e) {
            return null;
        }
    }

    /**
     * 比较两个日期时间的大小,反回1表示preDateStr > nextDateStr，0就相等，-1为小于
     *
     * @param preDateStr
     * @param nextDateStr
     * @return result
     */
    public static int compareDate(Object preDateStr, Object nextDateStr) {
        int result = 0;
        Date preDate = parseDate(preDateStr);
        Date nextDate = parseDate(nextDateStr);
        try {
            result = preDate.compareTo(nextDate);
        } catch (Exception e) {
            result = 0;
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 获取过去的天数
     *
     * @param date
     * @return
     */
    public static long pastDays(Date date) {
        long t = System.currentTimeMillis() - date.getTime();
        return t / (24 * 60 * 60 * 1000);
    }

    /**
     * 获取过去的小时
     *
     * @param date
     * @return
     */
    public static long pastHour(Date date) {
        long t = System.currentTimeMillis() - date.getTime();
        return t / (60 * 60 * 1000);
    }

    /**
     * 获取过去的分钟
     *
     * @param date
     * @return
     */
    public static long pastMinutes(Date date) {
        long t = System.currentTimeMillis() - date.getTime();
        return t / (60 * 1000);
    }

    /**
     * 转换为时间（天,时:分:秒.毫秒）
     *
     * @param timeMillis
     * @return
     */
    public static String formatDateTime(long timeMillis) {
        long day = timeMillis / (24 * 60 * 60 * 1000);
        long hour = (timeMillis / (60 * 60 * 1000) - day * 24);
        long min = ((timeMillis / (60 * 1000)) - day * 24 * 60 - hour * 60);
        long s = (timeMillis / 1000 - day * 24 * 60 * 60 - hour * 60 * 60 - min * 60);
        long sss = (timeMillis - day * 24 * 60 * 60 * 1000 - hour * 60 * 60 * 1000 - min * 60 * 1000 - s * 1000);
        return (day > 0 ? day + "," : "") + hour + ":" + min + ":" + s + "." + sss;
    }

    /**
     * 获取某一天的开始时间（0点）
     *
     * @param date
     * @return
     */
    public static Date getDateStart(Date date) {
        if (date == null) {
            return null;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            date = sdf.parse(formatDate(date, "yyyy-MM-dd") + " 00:00:00");
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }

    /**
     * 获取某一天的结束时间(23:59)
     *
     * @param date
     * @return
     */
    public static Date getDateEnd(Date date) {
        if (date == null) {
            return null;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            date = sdf.parse(formatDate(date, "yyyy-MM-dd") + " 23:59:59");
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }

    /**
     * 获取某一天的前几天或者后几天，根据数字符号决定天数
     *
     * @param dateObj
     * @param days
     * @return
     */
    public static String getPastDayStr(Object dateObj, int days) {
        Date date = parseDate(dateObj);
        long time = date.getTime() + days * (long) (24 * 60 * 60 * 1000);
        return formatDate(new Date(time));
    }

    /**
     * preDateStr - nextDateStr 返回秒数
     *
     * @param preDateStr
     * @param nextDateStr
     * @return
     */
    public static long getSubSecond(Object preDateStr, Object nextDateStr) {
        Date preDate = parseDate(preDateStr);
        Date nextDate = parseDate(nextDateStr);
        long result = (preDate.getTime() - nextDate.getTime()) / 1000L;
        return result;
    }

    /**
     * 返回过去的天数： preDateStr - nextDateStr
     *
     * @param preDateStr
     * @param nextDateStr
     * @return
     */
    public static long getDifferDate(Object preDateStr, Object nextDateStr) {
        return getSubSecond(preDateStr, nextDateStr) / (60 * 60 * 24L);
    }


    public static Date getDateByString(String time) {
        Date date = null;
        if (time == null) {
            return date;
        }

        String pattern = "yyyy-MM-dd HH:mm:ss";
        SimpleDateFormat format = new SimpleDateFormat(pattern);
        try {
            date = format.parse(time);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }

    /**
     * @return 日期 xx天前 xx小时前||xx分钟前||xx秒前
     */
    public static String getShortTime(String time) {
        Date date = getDateByString(time);
        if (date == null) {
            return null;
        }
        return getShortTime(date);
    }

    public static String getShortTime(Date date) {
        if (date == null) {
            return null;
        }
        String shortString;
        long now = Calendar.getInstance().getTimeInMillis();
        long delTime = (now - date.getTime()) / 1000;
        if (delTime > 365 * 24 * 60 * 60) {
            shortString = (int) (delTime / (365 * 24 * 60 * 60)) + "年前";
        } else if (delTime > 24 * 60 * 60) {
            shortString = (int) (delTime / (24 * 60 * 60)) + "天前";
        } else if (delTime > 60 * 60) {
            shortString = (int) (delTime / (60 * 60)) + "小时前";
        } else if (delTime > 60) {
            shortString = (int) (delTime / (60)) + "分前";
        } else if (delTime > 1) {
            shortString = delTime + "秒前";
        } else {
            shortString = "刚刚";
        }
        return shortString;
    }

    public static Date convertDate(String dateStr, String dateFmt) {
        return convertDate(Locale.CHINA, dateStr, dateFmt);
    }

    public static Date convertDate(Locale locale, String dateStr, String dateFmt) {
        try {
            if (locale == null) {
                locale = Locale.getDefault();
            }
            SimpleDateFormat dateFmter = new SimpleDateFormat(dateFmt, locale);
            return dateFmter.parse(dateStr);
        } catch (Exception exception) {
            exception.printStackTrace();
            return null;
        }
    }

    /**
     * 获取日期天数差
     *
     * @param startDate
     * @param endDate
     * @return
     */
    public static int getDiffDays(Date startDate, Date endDate) {
        return (int) ((endDate.getTime() - startDate.getTime()) / (1000 * 3600 * 24));
    }

    public static int getDiffMonths(String startDate, String endDate) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
        Calendar bef = Calendar.getInstance();
        Calendar aft = Calendar.getInstance();
        try {
            bef.setTime(sdf.parse(startDate));
            aft.setTime(sdf.parse(endDate));
            int result = aft.get(Calendar.MONTH) - bef.get(Calendar.MONTH);
            int month = (aft.get(Calendar.YEAR) - bef.get(Calendar.YEAR)) * 12;
            return Math.abs(month + result);
        } catch (ParseException e) {
            e.printStackTrace();
            return 0;
        }
    }

    /**
     * 日期增加天数
     *
     * @param date
     * @param day
     * @return
     */
    public static Date addDayToDate(Date date, int day) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, day);
        return calendar.getTime();
    }

    /**
     * 日期增加月数
     *
     * @param date
     * @param month
     * @return
     */
    public static Date addMonthToDate(Date date, int month) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.MONTH, month);
        return calendar.getTime();
    }

    /**
     * 将字符串日期转换成DATE类型
     *
     * @param dateStr
     * @param dateFmt
     * @return
     * @throws ParseException
     */
    public static Date convertStringToDate(String dateStr, String dateFmt) {
        try {
            SimpleDateFormat sdf = new SimpleDateFormat(dateFmt);
            Date date = sdf.parse(dateStr);
            return date;
        } catch (ParseException exception) {
            exception.printStackTrace();
            return null;
        }
    }

    /**
     * 根据月周期开始时间 计算开始和结束的临界时间点
     *
     * @param startDay
     * @return
     */
    public static String[] getMonthCriticalValue(@Nonnull Integer startDay) {
        startDay = startDay < 1 ? 1 : startDay > 31 ? 31 : startDay;
        Calendar now = Calendar.getInstance();
        int day = now.get(Calendar.DAY_OF_MONTH);
        String monthStart, monthEnd;
        if (day >= startDay) {
            monthStart = DateFormatUtils.format(now.getTimeInMillis() - (long) (day - startDay) * 1000 * 60 * 60 * 24, "yyyy-MM-dd");
        } else {
            long lastMonthEndDayTime = org.apache.commons.lang3.time.DateUtils.truncate(now, Calendar.MONTH).getTimeInMillis() - 1;
            Calendar lastMonth = Calendar.getInstance();
            lastMonth.setTimeInMillis(lastMonthEndDayTime);
            int gap = lastMonth.get(Calendar.DAY_OF_MONTH) - startDay;
            monthStart = DateFormatUtils.format(lastMonthEndDayTime - (gap >= 0 ? (long) gap * 1000 * 60 * 60 * 24 : 0), "yyyy-MM-dd");
        }
        monthEnd = DateFormatUtils.format(now, "yyyy-MM-dd");
        return new String[]{monthStart, monthEnd};
    }

    public static void main(String[] args) throws InterruptedException {
        //  Thread.sleep(2000);
        //System.out.println(getShortTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date())));
        System.out.println(getMonthCriticalValue(1));
        System.out.println(new Date(1590548048641L));

        System.out.println(new Date(1592596615937L));


    }
}
