package com.common.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author 潘维吉
 * @date 2018-11-21 16:25
 * 公共工具类
 */
public class Utils {

    /**
     * 实体类对象转map对象方法
     *
     * @param object
     * @return
     */
    public static Map<String, Object> convertBeanToMap(Object object) {
        if (object instanceof Map) {
            return (Map) object;
        }
        Map<String, Object> map = new HashMap<>(10);
        //获得类的的属性名 数组
        Field[] fields = object.getClass().getDeclaredFields();
        try {
            for (Field field : fields) {
                field.setAccessible(true);
                String name = new String(field.getName());
                map.put(name, field.get(object));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return map;
    }

    /**
     * map对象转实体类对象方法
     *
     * @param map
     * @return
     */
    public static Object convertMapToBean(Map<String, Object> map, Object object) {
        BeanUtils.copyProperties(map, object);
        return object;
    }

    /**
     * 两个数组合并
     *
     * @param first
     * @param second
     * @param <T>
     * @return
     */
    public static <T> T[] arrayConcat(T[] first, T[] second) {
        return Stream.concat(
                Stream.of(first),
                Stream.of(second)
        ).toArray(i -> (T[]) Arrays.copyOf(new Object[0], i, first.getClass()));
    }

    /**
     * 差集  返回两个数组之间的差异
     * 从first里面查找与second里不同的数组
     */
    public static <T> T[] arrayDifference(T[] first, T[] second) {
        return (T[]) Arrays.stream(first)
                .filter(item -> !Arrays.asList(second).contains(item))
                .toArray();
    }

    /**
     * 将枚举转换为 Map，其中 key 是枚举名，value 是枚举本
     *
     * @param enumClass
     * @param <E>
     * @return
     */
    public static <E extends Enum<E>> Map<String, E> getEnumMap(final Class<E> enumClass) {
        Map map = Arrays.stream(enumClass.getEnumConstants())
                .collect(Collectors.toMap(Enum::name, Function.identity(), (oldValue, newValue) -> newValue));
        map.forEach((key, value) -> {
            try {
                map.put(key, JSON.parseObject(value.toString().replaceAll("'", "\"")));
            } catch (JSONException e) {
            }
        });
        return map;
    }

    /**
     * 合并两个javabean的属性值
     *
     * @param sourceBean
     * @param targetBean
     * @param <T>
     * @return
     */
    public static <T> T mergeBeans(T sourceBean, T targetBean) {
        Class sourceBeanClass = sourceBean.getClass();
        Class targetBeanClass = targetBean.getClass();
        Field[] sourceFields = sourceBeanClass.getDeclaredFields();
        Field[] targetFields = targetBeanClass.getDeclaredFields();

        for (int i = 0; i < sourceFields.length; i++) {
            Field sourceField = sourceFields[i];
            if (Modifier.isStatic(sourceField.getModifiers())) {
                continue;
            }
            Field targetField = targetFields[i];
            if (Modifier.isStatic(targetField.getModifiers())) {
                continue;
            }
            sourceField.setAccessible(true);
            targetField.setAccessible(true);
            try {
                if ((targetField.get(targetBean) != null)) {
                    sourceField.set(sourceBean, targetField.get(targetBean));
                }
            } catch (IllegalArgumentException | IllegalAccessException e) {
                e.printStackTrace();
            }
        }
        return sourceBean;
    }

    /**
     * 根据集合获取到id字符串 以逗号,分割
     */
    public static <T> String getIdsStr(List<T> list) {
        List<Map> mapList = list.stream().map(item -> convertBeanToMap(item)
        ).collect(Collectors.toList());
        return StringUtils.join(mapList.stream().map((item) -> item.get("id")).distinct().collect(Collectors.toList()), ",");
    }

    /**
     * 以小写字符串的形式获取操作系统的名称
     *
     * @return
     */
    public static String osName() {
        return System.getProperty("os.name").toLowerCase();
    }

    /**
     * 语义化版本号x.y.z大小比较
     *
     * @param version1
     * @param version2
     * @return 0代表相等，1代表version1大于version2，-1代表version1小于version2
     */
    public static int compareVersion(String version1, String version2) {
        if (version1.equals(version2)) {
            return 0;
        }
        String[] version1Array = version1.split("\\.");
        String[] version2Array = version2.split("\\.");
        int index = 0;
        // 获取最小长度值
        int minLen = Math.min(version1Array.length, version2Array.length);
        int diff = 0;
        // 循环判断每位的大小
        while (index < minLen
                && (diff = Integer.parseInt(version1Array[index])
                - Integer.parseInt(version2Array[index])) == 0) {
            index++;
        }
        if (diff == 0) {
            // 如果位数不一致，比较多余位数
            for (int i = index; i < version1Array.length; i++) {
                if (Integer.parseInt(version1Array[i]) > 0) {
                    return 1;
                }
            }

            for (int i = index; i < version2Array.length; i++) {
                if (Integer.parseInt(version2Array[i]) > 0) {
                    return -1;
                }
            }
            return 0;
        } else {
            return diff > 0 ? 1 : -1;
        }
    }


}
