package com.core.config;

import com.alibaba.fastjson.JSON;
import com.core.exception.ServiceException;
import com.core.result.Result;
import com.core.result.ResultCodeEnum;
import com.core.security.jwt.JwtInterceptor;
import com.core.util.LogsUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.method.annotation.MethodArgumentTypeMismatchException;
import org.springframework.web.servlet.HandlerExceptionResolver;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.NoHandlerFoundException;
import org.springframework.web.servlet.config.annotation.CorsRegistry;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import javax.annotation.Resource;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.StringJoiner;

/**
 * @author 潘维吉
 * @date 2018-05-20
 * MVC 核心统一配置
 * 统一异常验证处理、添加拦截器、CORS跨域解决等
 */
@Configuration
@Slf4j
public class MvcConfig implements WebMvcConfigurer {

    /**
     * 是否开启cors跨域 默认true开启
     */
    @Value("${project.cors.enabled:true}")
    private boolean corsEnabled;
    /**
     * cors跨域允许的源 默认*全部
     */
    @Value("${project.cors.allowed-origins:*}")
    private String allowedOrigins;
    /**
     * 激活的application.yaml文件 dev调试 prod生产模式  默认值:prod
     */
    @Value("${spring.profiles.active:prod}")
    private String env;

    /**
     * JWT鉴权拦截器
     */
    @Resource
    private JwtInterceptor jwtInterceptor;
    /**
     * 日志工具类处理
     */
    @Resource
    private LogsUtils logsUtil;

    /**
     * 统一异常处理
     */
    @Override
    public void configureHandlerExceptionResolvers(List<HandlerExceptionResolver> exceptionResolvers) {
        exceptionResolvers.add((request, response, handler, exception) -> {
            //统一API响应结果封装类
            Result result = new Result();
            //请求URI
            String requestURI = request.getRequestURI();
            //请求方法
            String requestMethod = request.getMethod();

            //信息请求路径组合
            String msg = "RESTful API接口 [ " + requestURI + " ] ";
            //异常信息
            String errorMsg = exception.getMessage() == null ? "null空指针异常" : exception.getMessage();

            //是否要统一异常处理
            Boolean isHandle = true;
            //是否打印和存储ServiceException异常日志
            Boolean isLog = true;
            //异常代码
            String errorCode = null;

            // 400 业务失败的异常 401 jwt授权失败等，如“账号或密码错误”
            if (exception instanceof ServiceException) {
                ServiceException ex = (ServiceException) exception;
                isHandle = ex.getIsHandle();
                if (!isHandle) {
                    return new ModelAndView();
                }
                isLog = ex.getIsLog();
                // 响应码枚举码
                ResultCodeEnum resultCode = ex.getCode();
                result.setCode(resultCode == null ? ResultCodeEnum.FAIL : resultCode);
                msg = msg + errorMsg;
            }
            // 400 传入参数验证错误异常
            else if (exception instanceof BindException
                    || exception instanceof MethodArgumentNotValidException
                    || exception instanceof ConstraintViolationException
                    || exception instanceof MethodArgumentTypeMismatchException
                    || exception instanceof HttpMessageNotReadableException) {
                result.setCode(ResultCodeEnum.FAIL);
                msg = msg + "参数校验失败";
                StringJoiner paramsErrors = new StringJoiner(", ");
                // 遍历所有参数验证错误  统一返回给前端
                List<FieldError> fieldErrorList = new ArrayList<>();
                if (exception instanceof BindException) {
                    fieldErrorList = ((BindException) exception).getBindingResult().getFieldErrors();
                } else if (exception instanceof MethodArgumentNotValidException) {
                    fieldErrorList = ((MethodArgumentNotValidException) exception).getBindingResult().getFieldErrors();
                } else if (exception instanceof ConstraintViolationException) {
                    paramsErrors.add("参数验证失败: " + exception.getMessage());
                } else if (exception instanceof MethodArgumentTypeMismatchException) {
                    paramsErrors.add("参数类型不匹配: " + exception.getMessage());
                } else {
                    paramsErrors.add("参数异常: " + exception.getMessage());
                }
                for (FieldError fieldError : fieldErrorList) {
                    paramsErrors.add(fieldError.getField() + fieldError.getDefaultMessage());
                }
                errorMsg = paramsErrors.toString();
            }
            // 404 找不到资源
            else if (exception instanceof NoHandlerFoundException) {
                result.setCode(ResultCodeEnum.NOT_FOUND);
                msg = msg + "不存在";
                // 业务api接口有效 排除网络爬虫 服务器自身检测 静态资源等访问的无效错误
                if (!requestURI.startsWith("/api")) {
                    errorMsg = "";
                }
            }
            // 404 Servlet异常 在NoHandlerFoundException之后处理
            else if (exception instanceof ServletException) {
                result.setCode(ResultCodeEnum.NOT_FOUND);
                msg = msg + errorMsg;
            }
            // 500服务器端异常
            else {
                result.setCode(ResultCodeEnum.INTERNAL_SERVER_ERROR);
                errorCode = initErrorCode();
                msg = msg + "服务器端异常";
            }
            // 不同环境 不同异常类型 返回不同信息
            String genMsg = getExceptionMsg(msg, errorMsg, errorCode);
            result.setMsg(genMsg);
            // response响应捕获的异常结果返回客户端 状态码200
            responseResult(response, result);

            if (isLog && result.getCode() != ResultCodeEnum.NOT_FOUND.getCode()) { // 404 不打印日志
                // 异常信息日志统一组合  打印和存储
                logsUtil.exceptionLogs(handler, exception, result, requestMethod, request, genMsg, errorCode);
            }
            return new ModelAndView();
        });
    }

    /**
     * 解决CORS跨域问题
     */
    @Override
    public void addCorsMappings(CorsRegistry registry) {
        if (corsEnabled) {
            registry.addMapping("/**") //设置允许跨域的路径 **代表所有路径 如 /api/**
                    .allowedOrigins(allowedOrigins) //设置允许跨域请求的域名 *代表所有,可以使用指定的ip,多个的话可以用逗号分隔,默认为*
                    .allowCredentials(true) //是否允许证书 支持证书 默认为true
                    .allowedMethods("OPTIONS", "GET", "POST", "PUT", "PATCH", "DELETE") //设置允许的请求方法
                    .maxAge(3600) //跨域允许时间 最大过期时间 默认为-1
                    .allowedHeaders("*"); //JWT授权
        }
    }

    /**
     * 添加注册拦截器
     */
    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        //多个拦截器时 以此添加 执行顺序按添加顺序
        //接口签名认证拦截器 使用Json Web Token
        registry.addInterceptor(jwtInterceptor);
        // .addPathPatterns("/api/*"); //指定拦截正则匹配模式, 限制拦截器拦截请求
    }

    /**
     * 响应结果处理
     */
    private void responseResult(HttpServletResponse response, Result result) {
        response.setCharacterEncoding("UTF-8");
        response.setHeader("Content-type", "application/json;charset=UTF-8");
        //401认证 403授权问题 可修改http返回状态码 默认业务异常都是200
        if (result.getCode() == ResultCodeEnum.UNAUTHORIZED.getCode()) {
            response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
        } else if (result.getCode() == ResultCodeEnum.FORBIDDEN.getCode()) {
            response.setStatus(HttpServletResponse.SC_FORBIDDEN);
        } else {
            response.setStatus(HttpServletResponse.SC_OK);
        }
        try {
            response.getWriter().write(JSON.toJSONString(result));
        } catch (IOException e) {
            log.error(e.getMessage());
        }
    }

    /**
     * 根据不同异常不同环境生成不同的日志和响应前端的msg信息
     * 生成唯一日志标识 可用于ELK等搜索使用
     */
    private String getExceptionMsg(String message, String errorMsg, String errorCode) {
        if (errorCode != null) {
            String msg = "请复制或截屏标识码给系统管理员: " + errorCode;
            return msg;
        } else {
            message = errorMsg == "" ? message : errorMsg;
            return message;
        }
    }

    private String initErrorCode() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyMMddHHmmss");
        return sdf.format(new Date()) + (int) ((Math.random() * 9 + 1) * 1000);
    }

}
