package com.core.security.jwt;

import com.alibaba.fastjson.JSON;
import com.core.exception.ServiceException;
import com.core.result.ResultCodeEnum;
import io.jsonwebtoken.Claims;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author 潘维吉
 * @date 2018-05-20
 * JWT自定义拦截器，判断此次请求是否有token权限
 * <p>
 * preHandle在业务处理器处理请求之前被调用
 * postHandle在业务处理器处理请求执行完成后,生成视图之前执行
 * afterCompletion在DispatcherServlet完全处理完请求后即在视图渲染完成后回调
 */
@Component
@Slf4j
public class JwtInterceptor implements HandlerInterceptor {

    /** 是否开启jwt认证 默认true开启 */
    @Value("${project.jwt.enabled:true}")
    private boolean JwtEnabled;

    /**
     * 请求前预处理
     * return false 则请求中断
     */
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        if (!JwtEnabled) {
            return true;
        }
        // 拦截获取http请求的URI地址和方法
        String uri = request.getRequestURI();
        String uriMethod = request.getMethod();
        // 需要进行跨域请求的资源api 当服务端检测到OPTIONS请求时候放行
       /* if ("OPTIONS".equals(uriMethod)) {
            response.setStatus(HttpStatus.SC_OK);
            return true;
        }*/
        // 不处理授权  如果不是映射到方法直接通过  自定义不授权路径直接通过  HandlerMethod及子类主要用于封装方法调用相关信息
        if (!(handler instanceof HandlerMethod) || JwtPathProperties.isAuth(uri, uriMethod)) {
            return true;
        }

        String authorizationHeader = JwtManager.AUTHORIZATION_HEADER;
        String bearer = JwtManager.BEARER;
        // 从header中得到token
        String authorization = request.getHeader(authorizationHeader);
        // 返回401错误
        if (authorization == null || !authorization.startsWith(bearer)) {
            throw new ServiceException(ResultCodeEnum.UNAUTHORIZED,
                    "授权" + authorizationHeader + "为空或无" + bearer + "前缀");
        }

        try {
            // 解析验证token
            Claims claims = JwtManager.parseToken(authorization.replaceAll(bearer, ""));
            if (claims != null) {
                // 如果token验证成功，将token对应的用户id
                Object userId = JSON.parseObject(claims.getSubject()).get("userId");
                // 如果token验证成功，将token对应的用户id存在request中，便于之后注入
                request.setAttribute("CURRENT_USER_ID", userId);
                /*log.info("JwtInterceptor当前请求用户id=" + userId);*/
                return true;
            }
        } catch (Exception e) {
            // 如果验证token失败，并且方法注明了Authorization，返回401错误
            throw new ServiceException(ResultCodeEnum.UNAUTHORIZED, "验证token授权认证失败或已过期");
        }
        // claims为null情况
        throw new ServiceException(ResultCodeEnum.UNAUTHORIZED, "token解析异常");
    }
}
